* wwDotNetBridge.DataSetToCursors converts a DataSet into cursors but it's too
* slow when there are a lot of records because it converts the DataSet to XML
* and then uses XMLAdapter to create a cursor, and XMLAdapter is really slow
* when there are lots of records. So, we'll take a different approach: create a
* cursor with the same structure as the specified DataTable and fill the cursor
* with the contents of each row.

lparameters toBridge, ;
	toTable, ;
	tcCursor, ;
	tnRecords
local lnColumns, ;
	lcCursor, ;
	laColumns[1], ;
	laMaxLength[1], ;
	lnI, ;
	loColumn, ;
	lcColumnName, ;
	lcDataType, ;
	lcType, ;
	lcAlias, ;
	lnRecords, ;
	lnRows, ;
	loRow, ;
	lnJ, ;
	luValue

* Figure out how many columns there are.

lnColumns = toBridge.GetPropertyEx(toTable, 'Columns.Count')

* Store each column name in an array and create a CREATE CURSOR statement.

lcCursor = ''
dimension laColumns[lnColumns, 2], laMaxLength[lnColumns]
laMaxLength = 0
for lnI = 0 to lnColumns - 1
	loColumn     = toBridge.GetPropertyEx(toTable, 'Columns[' + transform(lnI) + ']')
	lcColumnName = loColumn.ColumnName
	lcDataType   = toBridge.GetPropertyEx(loColumn.DataType, 'Name')
	do case
		case inlist(lcDataType, 'Decimal', 'Double', 'Single')
			lcType = 'B(8)'
		case lcDataType = 'Boolean'
			lcType = 'L'
		case lcDataType = 'DateTime'
			lcType = 'T'
		case lcDataType = 'Int'
			lcType = 'I'
		case loColumn.MaxLength = -1
			lcType = 'M'
		otherwise
			lcType = 'C (' + transform(loColumn.MaxLength) + ')'
	endcase
	lcCursor = lcCursor + iif(empty(lcCursor), '', ',') + lcColumnName + ;
		' ' + lcType + ' null'
	laColumns[lnI + 1, 1] = lcColumnName
	laColumns[lnI + 1, 2] = left(lcType, 1)
	local &lcColumnName
next lnI

* Create the cursor.

lcAlias  = sys(2015)
lcCursor = 'create cursor ' + lcAlias + ' (' + lcCursor + ')'
&lcCursor

* Go through each row, get each column, and populate the cursor.

lnRecords = evl(tnRecords, 9999999999)
lnRows    = toBridge.GetPropertyEx(toTable, 'Rows.Count')
for lnI = 0 to min(lnRows, lnRecords) - 1
	loRow = toBridge.GetPropertyEx(toTable, 'Rows[' + transform(lnI) + ']')
	for lnJ = 0 to lnColumns - 1
		luValue = toBridge.GetPropertyEx(loRow, 'ItemArray[' + transform(lnJ) + ']')
		store luValue to (laColumns[lnJ + 1, 1])
		if vartype(luValue) = 'C' or laColumns[lnJ + 1, 2] $ 'CM'
			luValue = transform(luValue)
			laMaxLength[lnJ + 1] = max(laMaxLength[lnJ + 1], len(luValue))
		endif vartype(luValue) = 'C' ...
	next lnJ
	insert into (lcAlias) from memvar
next lnI

* Do a final select to get the correct column lengths.

lcCursor = ''
for lnI = 1 to alen(laColumns, 1)
	lcColumnName = laColumns[lnI, 1]
	lcType       = laColumns[lnI, 2]
	if lcType $ 'CM' and laMaxLength[lnI] < 255
		lcType = 'C (' + transform(max(laMaxLength[lnI], 1)) + ')'
	endif lcType $ 'CM' ...
	lcCursor = lcCursor + iif(empty(lcCursor), '', ',') + lcColumnName + ;
		' ' + lcType + ' null'
next lnI
lcCursor = 'create cursor ' + tcCursor + ' (' + lcCursor + ')'
&lcCursor
append from dbf(lcAlias)
use in (lcAlias)
go top
return
